********************************************************************************
*                                                                              *
*      Input file for the Lagrangian particle dispersion model FLEXPART        *
*                       Please specify your NESTED output grid                 *
*      Dominik Brunner: This version is for output on COSMO2 domain            *
********************************************************************************

2.  ------.----       4X,F11.4
    {{ data.longitude_of_output_grid }}        LONGITUDE OF LOWER LEFT CORNER OF OUTPUT GRID
    OUTLONLEFT        (left boundary of the first grid cell - not its centre)

3.  ------.----       4X,F11.4
    {{ data.latitude_of_output_grid }}       LATITUDE OF LOWER LEFT CORNER OF OUTPUT GRID
    OUTLATLOWER       (lower boundary of the first grid cell - not its centre)

4.  -----             4X,I5
    {{ data.number_of_grid_points_x }}            n NUMBER OF GRID POINTS IN X DIRECTION (= No. of cells + 1)
    NUMXGRID

5.  -----             4X,I5
    {{ data.number_of_grid_points_y }}             NUMBER OF GRID POINTS IN Y DIRECTION (= No. of cells + 1)
    NUMYGRID

6.  ------.---        4X,F10.3
    {{ data.grid_distance_x }}         GRID DISTANCE IN X DIRECTION
    DXOUTLON

7.  ------.---        4X,F10.3
    {{ data.grid_distance_y }}         GRID DISTANCE IN Y DIRECTION
    DYOUTLAT
