********************************************************************************
*                                                                              *
*      Input file for the Lagrangian particle dispersion model FLEXPART        *
*                           Please select your options                         *
*                                                                              *
********************************************************************************

1. __                3X, I2
   {{ data.simulation_direction }}
   LDIRECT           1 FOR FORWARD SIMULATION, -1 FOR BACKWARD SIMULATION

2. ________ ______   3X, I8, 1X, I6
   {{ data.simulation_beginning_date[0] }} {{ data.simulation_beginning_date[1] }}
   YYYYMMDD HHMISS   BEGINNING DATE OF SIMULATION

3. ________ ______   3X, I8, 1X, I6
   {{ data.simulation_ending_date[0] }} {{ data.simulation_ending_date[1] }}
   YYYYMMDD HHMISS   ENDING DATE OF SIMULATION

4. _____             3X, I5
   {{ data.output_every_seconds }}
   SSSSS             OUTPUT EVERY SSSSS SECONDS

5. _____             3X, I5
   {{ data.time_average_of_output_seconds }}
   SSSSS             TIME AVERAGE OF OUTPUT (IN SSSSS SECONDS)

6. _____             3X, I5
   {{ data.sampling_rate_of_output }}
   SSSSS             SAMPLING RATE OF OUTPUT (IN SSSSS SECONDS)

7. _________         3X, I9
   {{ data.particle_splitting_time_constant }}
   SSSSSSSSS         TIME CONSTANT FOR PARTICLE SPLITTING (IN SECONDS)

8. _____             3X, I5
   {{ data.synchronisation_interval }}
   SSSSS             SYNCHRONISATION INTERVAL OF FLEXPART (IN SECONDS)

9.  ---.--           4X, F6.4
    {{ data.smaller_than_tl_factor }}
    CTL              FACTOR, BY WHICH TIME STEP MUST BE SMALLER THAN TL

10. ---              4X, I3
    {{ data.vertical_motion_time_decrease }}
    IFINE            DECREASE OF TIME STEP FOR VERTICAL MOTION BY FACTOR IFINE

11. -                4X, I1
    {{ data.concentration_output }}
    IOUT             1 CONCENTRATION (RESIDENCE TIME FOR BACKWARD RUNS) OUTPUT, 2 MIXING RATIO OUTPUT, 3 BOTH,4 PLUME TRAJECT., 5=1+4

12. -                4X, I1
    {{ data.particle_dump }}
    IPOUT            PARTICLE DUMP: 0 NO, 1 EVERY OUTPUT INTERVAL, 2 ONLY AT END, 4 WHEN LEAVING DOMAIN

13. _                4X, I1
    {{ 1 if data.subgrid_terrain_effect_parameterization else 0 }}
    LSUBGRID         SUBGRID TERRAIN EFFECT PARAMETERIZATION: 1 YES, 0 NO

14. _                4X, I1
    {{ data.convection_parametrization }}
    LCONVECTION      CONVECTION: 2 TIEDTKE, 1 EMANUEL, 0 NO

15. _                4X, I1
    {{ 1 if data.age_spectra else 0 }}
    LAGESPECTRA      AGE SPECTRA: 1 YES, 0 NO

16. _                4X, I1
    {{ 1 if data.dumped_particle_data else 0 }}
    IPIN             CONTINUE SIMULATION WITH DUMPED PARTICLE DATA: 1 YES, 0 NO

17. _                4X,I1
    {{ 1 if data.output_for_each_release else 0 }}
    IOFR             IOUTPUTFOREACHREL CREATE AN OUPUT FILE FOR EACH RELEASE LOCATION: 1 YES, 0 NO

18. _                4X, I1
    {{ 1 if data.calculate_fluxes else 0 }}
    IFLUX            CALCULATE FLUXES: 1 YES, 0 NO

19. _                4X, I1
    {{ data.domain_filling_trajectory }}
    MDOMAINFILL      DOMAIN-FILLING TRAJECTORY OPTION: 1 YES, 0 NO, 2 STRAT. O3 TRACER

20. _                4X, I1
    {{ data.concentration_units_at_source }}
    IND_SOURCE       1=MASS UNIT , 2=MASS MIXING RATIO UNIT

21. _                4X, I1
    {{ data.concentration_units_at_receptor }}
    IND_RECEPTOR     1=MASS UNIT , 2=MASS MIXING RATIO UNIT

22. _                4X, I1
    {{ 1 if data.quasilagrangian_mode_to_track_particles else 0 }}
    MQUASILAG        QUASILAGRANGIAN MODE TO TRACK INDIVIDUAL PARTICLES: 1 YES, 0 NO

23. _                4X, I1
    {{ 1 if data.nested_output else 0 }}
    NESTED_OUTPUT    SHALL NESTED OUTPUT BE USED? 1 YES, 0 NO

24. _                4X, I1
    {{ 1 if data.cosmo_model_mixing_height else 0 }}
    LCOSMO_HMIX      SHALL COSMO MODEL MIXING HEIGHT BE USED IF PRESENT? 1 YES, 0 NO

25. ---.--           4X, F6.2
    {{ data.cosmo_grid_relaxation_zone_width }}
    RLWIDTH          WIDTH OF COSMO GRID RELAXATION ZONE IN KM

1. Simulation direction, 1 for forward, -1 for backward in time
	(consult Seibert and Frank, 2004 for backward runs)

2. Beginning date and time of simulation. Must be given in format
   YYYYMMDD HHMISS, where YYYY is YEAR, MM is MONTH, DD is DAY, HH is HOUR,
   MI is MINUTE and SS is SECOND. Current version utilizes UTC.

3. Ending date and time of simulation. Same format as 3.

4. Average concentrations are calculated every SSSSS seconds.

5. The average concentrations are time averages of SSSSS seconds
   duration. If SSSSS is 0, instantaneous concentrations are outputted.

6. The concentrations are sampled every SSSSS seconds to calculate the time
   average concentration. This period must be shorter than the averaging time.

7. Time constant for particle splitting. Particles are split into two
   after SSSSS seconds, 2xSSSSS seconds, 4xSSSSS seconds, and so on.

8. All processes are synchronized with this time interval (lsynctime).
   Therefore, all other time constants must be multiples of this value.
   Output interval and time average of output must be at least twice lsynctime.

9. CTL must be >1 for time steps shorter than the  Lagrangian time scale
   If CTL<0, a purely random walk simulation is done

10.IFINE=Reduction factor for time step used for vertical wind

11.IOUT determines how the output shall be made: concentration
   (ng/m3, Bq/m3), mixing ratio (pptv), or both, or plume trajectory mode,
   or concentration + plume trajectory mode.
   In plume trajectory mode, output is in the form of average trajectories.

12.IPOUT determines whether particle positions are outputted (in addition
   to the gridded concentrations or mixing ratios) or not.
   0=no output, 1 output every output interval, 2 only at end of the
   simulation

13.Switch on/off subgridscale terrain parameterization (increase of
   mixing heights due to subgridscale orographic variations)

14.Switch on/off the convection parameterization

15.Switch on/off the calculation of age spectra: if yes, the file AGECLASSES
   must be available

16. If IPIN=1, a file "partposit_end" from a previous run must be available in
    the output directory. Particle positions are read in and previous simulation
    is continued. If IPIN=0, no particles from a previous run are used

17. IF IOUTPUTFOREACHRELEASE is set to 1, one output field for each location
    in the RLEASE file is created. For backward calculation this should be
    set to 1. For forward calculation both possibilities are applicable.

18. If IFLUX is set to 1, fluxes of each species through each of the output
    boxes are calculated. Six fluxes, corresponding to northward, southward,
    eastward, westward, upward and downward are calculated for each grid cell of
    the output grid. The control surfaces are placed in the middle of each
    output grid cell. If IFLUX is set to 0, no fluxes are determined.

19. If MDOMAINFILL is set to 1, the first box specified in file RELEASES is used
    as the domain where domain-filling trajectory calculations are to be done.
    Particles are initialized uniformly distributed (according to the air mass
    distribution) in that domain at the beginning of the simulation, and are
    created at the boundaries throughout the simulation period.

20. IND_SOURCE switches between different units for concentrations at the source
    NOTE that in backward simulations the release of computational particles
    takes place at the "receptor" and the sampling of particles at the "source".
          1=mass units (for bwd-runs = concentration)
          2=mass mixing ratio units
21. IND_RECEPTOR switches between different units for concentrations at the receptor
          1=mass units (concentrations)
          2=mass mixing ratio units

22. MQUASILAG indicates whether particles shall be numbered consecutively (1) or
    with their release location number (0). The first option allows tracking of
    individual particles using the partposit output files

23. NESTED_OUTPUT decides whether model output shall be made also for a nested
    output field (normally with higher resolution)

24. COSMO_HMIX decides whether the mixing layer height field eventually provided
    in the GRIB files is used instead of the internally calculated field.
    Mixing layer heights are stored as MIX_LAY_DE, DWD table 2, GBRIB code 67

25. RLWIDTH is the width [km] of the relaxation zone in the COSMO model (as
    defined in the NAMELIST, but not specified in the GRIB output).
    Here, the width needs to be provided in km. COSMO-FLEXPART uses RLWIDTH
    to compute a parameter IRWIDTH which specifies the number of grid cells
    from the border at which a particle is transferred to the next outer nest
    or terminated if it is in the mother domain.
